/******************************************************************

Program reformats SFMR surface wind data for use with H*WIND.
("LF format"). To compile, use any ANSI c compile (such as gcc) 
and link with the math library, e.g.:

> gcc -o tolf tolf.c -lm

After successful compilation, 'tolf' is run with 2 command line arguments. 
The first argument is the SFMR file, and the second is an inflow 
angle adjustment to apply to the flight level wind direction, e.g.:

> tolf 20021002I.sfmr 20

Two files are generated by running 'tolf', one surface wind file, and one
flight-level wind file. The files are named the same as the input SFMR file,
with a '_sfc.lf' appended for the surface file, and a '_flt.lf' for 
the flight-level file. Again, the surface wind direction is just the 
flight-level direction with a fictitious frictional inflow angle adjustment. 
Currently, data is spit out at 30 second intervals, but this can be easily
modified.

Questions or comments, contact Eric.Uhlhorn@noaa.gov

***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define DTOR 3.14159265/180.0

void SpdDir2UV(float,float,float *,float *);

int main(int argc, char *argv[]) {

   FILE *fptr,*fsfc,*fflt;
   char date[8],sfcfile[50],fltfile[50];
   int  time,cent,year,month,day;
   float lon,lat,sws,fws,fwdir,alt,pres,rad,az;
   float inflow,u,v;

   if (argc != 3) {
      printf("Usage: tolf <infile> <inflow ang>\n");
      return -1;
   }

   inflow = atof(argv[2]);
   
/* Open and read sfmr data file */
   if ((fptr = fopen(argv[1],"r")) == NULL) {
      printf("Unable to open %s for read.\n",argv[1]);
      return -1;
   }

/* Create output filename strings */
   strcpy(fltfile,argv[1]);
   strcpy(sfcfile,argv[1]);
   strcat(fltfile,"_flt.lf");
   strcat(sfcfile,"_sfc.lf");

/* Open output files */
   if ((fsfc = fopen(sfcfile,"w")) == NULL) {
      printf("Unable to open %s for write.\n",sfcfile);
      return -1;
   }
   if ((fflt = fopen(fltfile,"w")) == NULL) {
      printf("Unable to open %s for write.\n",fltfile);
      return -1;
   }   


   while (fscanf(fptr,"%s %d %f %f %f %f %f %f %f %f %f",
      date,&time,&lon,&lat,&alt,&pres,&rad,&az,&sws,&fws,&fwdir)
      != EOF) {

            lon = -1.0*lon;	 
	    sscanf(date,"%2d%2d%2d%2d",&cent,&year,&month,&day);

// Write flight-level 
// Print every 30 secs.

         if ((time%100 == 0) || (time%100 == 30)) {

	    SpdDir2UV(fws,fwdir,&u,&v);		// Convert to UV

fprintf(fflt," 3 %.2d%.2d%.2d. %.6d %7.3f %7.3f %6.1f  -99.0  -99.0  -99.0 %6.1f %6.1f N430\n",
year,month,day,time,lat,lon,pres,u,v);

         }


// Add frictional inflow angle
	 fwdir = fwdir - inflow;
	 if (fwdir < 0.0)
	    fwdir += 360.0;

         if (sws > 0.0) {
	    SpdDir2UV(sws,fwdir,&u,&v);

// Write surface file
            if ((time%100 == 0) || (time%100 == 30))  /* Print every 30 secs. */

fprintf(fsfc," 7 %.2d%.2d%.2d. %.6d %7.3f %7.3f 1070.0  -99.0  -99.0  -99.0 %6.1f %6.1f S431\n",
year,month,day,time,lat,lon,u,v);

         }
        

   }
   return 0;

}


void SpdDir2UV(float spd, float dir, float *u, float *v) {
  
   dir = 270.0-dir;
   
   *u = spd*cos(dir*DTOR);
   *v = spd*sin(dir*DTOR);
   
}
